@@ -2,24 +2,44 @@
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.management import call_command
-from django.db import models
+from django.db import models, router
 from django.db.utils import IntegrityError
+from django.utils.translation import ugettext
 
-from mutant.models.model import (ModelDefinition, OrderingFieldDefinition,
-    UniqueTogetherDefinition, BaseDefinition)
 from mutant.contrib.text.models import CharFieldDefinition
 from mutant.contrib.related.models import ForeignKeyDefinition
 from mutant.db.models import MutableModel
-from mutant.tests.models.utils import BaseModelDefinitionTestCase
+from mutant.models.model import (ModelDefinition, OrderingFieldDefinition,
+    UniqueTogetherDefinition, BaseDefinition)
+from mutant.tests.models.utils import (BaseModelDefinitionTestCase,
+    skipUnlessMutantModelDBFeature)
 
 
+try:
+    raise ImportError()
+    from test.test_support import captured_stderr
+except ImportError:
+    # python 2.6 doesn't provide this helper
+    from contextlib import contextmanager
+    import StringIO
+    import sys
+
+    @contextmanager
+    def captured_stderr():
+        stderr = sys.stderr
+        try:
+            sys.stderr = StringIO.StringIO()
+            yield sys.stderr
+        finally:
+            sys.stderr = stderr
+
 class ModelDefinitionManipulationTest(BaseModelDefinitionTestCase):
     
     def test_model_class_creation_cache(self):
-        Member = self.model_def.model_class()
-        self.assertEqual(Member, self.model_def.model_class())
+        Model = self.model_def.model_class()
+        self.assertEqual(Model, self.model_def.model_class())
         
-        self.assertNotEqual(Member, self.model_def.model_class(force_create=True))
+        self.assertNotEqual(Model, self.model_def.model_class(force_create=True))
 
     def test_rename_model(self):
         """
@@ -29,21 +49,23 @@ class ModelDefinitionManipulationTest(BaseModelDefinitionTestCase):
         def get_table_name():
             return self.model_def.model_class()._meta.db_table
         
+        db = router.db_for_read(self.model_def.model_class())
+        
         table_name = get_table_name()
         self.model_def.app_label = 'myapp'
         self.model_def.save()
-        self.assertTableDoesntExists(table_name)
+        self.assertTableDoesntExists(db, table_name)
         table_name = get_table_name()
-        self.assertTableExists(table_name)
+        self.assertTableExists(db, table_name)
         
         self.model_def.object_name = 'MyModel'
         self.model_def.save()
-        self.assertTableDoesntExists(table_name)
+        self.assertTableDoesntExists(db, table_name)
         table_name = get_table_name()
-        self.assertTableExists(table_name)
+        self.assertTableExists(db, table_name)
         
         self.model_def.delete()
-        self.assertTableDoesntExists(table_name)
+        self.assertTableDoesntExists(db, table_name)
         
     def test_fixture_loading(self):
         call_command('loaddata', 'fixture_loading_test', verbosity=0, commit=False)
@@ -51,6 +73,31 @@ class ModelDefinitionManipulationTest(BaseModelDefinitionTestCase):
                                                 object_name='MyFixtureModel')
         Model = model_def.model_class()
         Model.objects.create()
+        
+    def test_verbose_name(self):
+        Model = self.model_def.model_class()
+        
+        self.assertEqual(Model._meta.verbose_name, self.model_def.model)
+        self.assertEqual(Model._meta.verbose_name_plural, "%ss" % self.model_def.model)
+        
+        self.model_def.verbose_name = u'MyMoDeL'
+        self.model_def.verbose_name_plural = u'MyMoDeLZ0Rs'
+        self.model_def.save()
+        
+        self.assertEqual(Model._meta.verbose_name, ugettext(u'MyMoDeL'))
+        self.assertEqual(Model._meta.verbose_name_plural, ugettext(u'MyMoDeLZ0Rs'))
+        
+    def test_multiple_model_definition(self):
+        """
+        Make sure multiple model definition can coexists
+        """
+        other_model_def = ModelDefinition.objects.create(app_label='app',
+                                                         object_name='OtherModel')
+        
+        self.assertNotEqual(other_model_def.model_class(),
+                            self.model_def.model_class())
+        
+        self.assertNotEqual(other_model_def.model_ct, self.model_def.model_ct)
             
 class ModelValidationTest(BaseModelDefinitionTestCase):
     
@@ -76,6 +123,8 @@ class ModelClassProxyProxyTests(BaseModelDefinitionTestCase):
         
         assert issubclass(Model, models.Model)
         
+        assert issubclass(Model, MutableModel)
+        
         assert unicode(Model) == u"<class 'mutant.apps.app.models.Model'>"
         
         assert sergei == Model.objects.get(name='Sergei')
@@ -101,10 +150,11 @@ class ModelClassProxyProxyTests(BaseModelDefinitionTestCase):
                                            name="name", max_length=10)
         
         Model = self.model_def.model_class()
+        db = router.db_for_write(Model)
         instance = Model.objects.create(name="Quebec")
         table_name = Model._meta.db_table
         self.model_def.delete()
-        self.assertTableDoesntExists(table_name)
+        self.assertTableDoesntExists(db, table_name)
         
         with self.assertRaises(AttributeError):
             Model(name="name")
@@ -154,6 +204,7 @@ class OrderingDefinitionTest(BaseModelDefinitionTestCase):
             ordering.lookup = 'f2__higgs_boson'
             ordering.clean()
 
+    @skipUnlessMutantModelDBFeature('supports_joins')
     def test_simple_ordering(self):
         Model = self.model_def.model_class()
         model_ct = ContentType.objects.get_for_model(Model) #app
@@ -197,6 +248,7 @@ class OrderingDefinitionTest(BaseModelDefinitionTestCase):
                                  transform=lambda x: x['f1'], ordered=True)
         f2_ordering.delete()
     
+    @skipUnlessMutantModelDBFeature('supports_joins')
     def test_multiple_ordering(self):
         Model = self.model_def.model_class()
         model_ct = ContentType.objects.get_for_model(Model) #app
@@ -268,10 +320,6 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         other_model_def = ModelDefinition.objects.create(app_label='app',
                                                          object_name='OtherModel')
         
-        #TODO: move that somewhere else
-        assert other_model_def.model_class() != self.model_def.model_class()
-        assert other_model_def.model_ct != self.model_def.model_ct
-        
         f2 = CharFieldDefinition.objects.create(model_def=other_model_def,
                                                 name='f2', max_length=25)
         self.ut.field_defs.add(self.f1, f2)
@@ -286,8 +334,9 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         """
         self.Model.objects.create(f1='a', f2='b')
         self.Model.objects.create(f1='a', f2='b')
-        with self.assertRaises(IntegrityError):
-            self.ut.field_defs.add(self.f1, self.f2)
+        with captured_stderr():
+            with self.assertRaises(IntegrityError):
+                self.ut.field_defs.add(self.f1, self.f2)
     
     def test_cannot_insert_duplicate_row(self):
         """
@@ -295,8 +344,9 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         """
         self.Model.objects.create(f1='a', f2='b')
         self.ut.field_defs.add(self.f1, self.f2)
-        with self.assertRaises(IntegrityError):
-            self.Model.objects.create(f1='a', f2='b')
+        with captured_stderr():
+            with self.assertRaises(IntegrityError):
+                self.Model.objects.create(f1='a', f2='b')
     
     def test_cannot_remove_unique(self):
         """
@@ -306,8 +356,9 @@ class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
         self.ut.field_defs.add(self.f1, self.f2)
         self.Model.objects.create(f1='a', f2='b')
         self.Model.objects.create(f1='a', f2='c')
-        with self.assertRaises(IntegrityError):
-            self.ut.field_defs.remove(self.f2)
+        with captured_stderr():
+            with self.assertRaises(IntegrityError):
+                self.ut.field_defs.remove(self.f2)
 
     def test_clear_removes_unique(self):
         """
@@ -428,5 +479,6 @@ class BaseDefinitionTest(BaseModelDefinitionTestCase):
         # This should cause the model to loose all it's fields and the table
         # to loose all it's columns
         bd.delete()
-        self.assertEqual(list(Model.objects.values_list()), [(1,), (2,)])
-        self.assertFieldDoesntExists(Model._meta.db_table, 'field')
+        self.assertEqual(list(Model.objects.values_list()),
+                         [(instance.id,) for instance in Model.objects.all()])
+        self.assertColumnDoesntExists(Model._meta.db_table, 'field')
